package com.fr.plugin;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;
import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.*;

import static org.apache.tomcat.util.codec.binary.Base64.encodeBase64String;

public class Signature {

    public static boolean checkSig(HttpServletRequest request, String sign) throws UnsupportedEncodingException {
        Map<String, String> paramMap = new HashMap<>();
        WPSConfig wpsConfig = WPSConfig.getInstance();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String name = parameterNames.nextElement();
            if (name.startsWith("_w_") && !name.equals("_w_signature")) {
                paramMap.put(name, request.getParameter(name));
            }
        }
        String signature = getSignature(paramMap, wpsConfig.getAppKey(),false);
        return sign.equals(signature);
    }

    public static String getSign(String name) throws UnsupportedEncodingException {
        Map<String, String> paramMap = new HashMap<>();
        WPSConfig wpsConfig = WPSConfig.getInstance();
        paramMap.put("_w_appid", wpsConfig.getAppid());
        paramMap.put("_w_userid", name);
        paramMap.put("_w_tokentype", "1");
        return getSignature(paramMap, wpsConfig.getAppKey(),true);
    }

    private static String getUrlParam(Map<String, String> params) throws UnsupportedEncodingException {
        StringBuilder builder = new StringBuilder();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            if (builder.length() > 0) {
                builder.append('&');
            }
            builder.append(URLEncoder.encode(entry.getKey(), "utf-8")).append('=').append(URLEncoder.encode(entry.getValue(), "utf-8"));
        }
        return builder.toString();
    }

    private static String getSignature(Map<String, String> params, String appSecret, boolean needEncode) {
        List<String> keys = new ArrayList();
        for (Map.Entry<String, String> entry : params.entrySet()) {
            keys.add(entry.getKey());
        }

        // 将所有参数按 key 的升序排序
        Collections.sort(keys, new Comparator<String>() {
            public int compare(String o1, String o2) {
                return o1.compareTo(o2);
            }
        });

        // 构造签名的源字符串
        StringBuilder contents = new StringBuilder("");
        for (String key : keys) {
            if (key == "_w_signature") {
                continue;
            }
            contents.append(key + "=").append(params.get(key));
            System.out.println("key:" + key + ",value:" + params.get(key));
        }
        contents.append("_w_secretkey=").append(appSecret);

        // 进行 hmac sha1 签名
        byte[] bytes = hmacSha1(appSecret.getBytes(), contents.toString().getBytes());
        // 字符串经过 Base64 编码
        String sign = encodeBase64String(bytes);
        if (needEncode) {
            try {
                sign = URLEncoder.encode(sign, "UTF-8");
            } catch (UnsupportedEncodingException e) {
                e.printStackTrace();
            }
            return sign;
        }
        return sign;
    }

    public static byte[] hmacSha1(byte[] key, byte[] data) {
        try {
            SecretKeySpec signingKey = new SecretKeySpec(key, "HmacSHA1");
            Mac mac = Mac.getInstance(signingKey.getAlgorithm());
            mac.init(signingKey);
            return mac.doFinal(data);
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        } catch (InvalidKeyException e) {
            e.printStackTrace();
        }
        return null;
    }

}