package com.fr.plugin.api;

import com.fr.decision.authority.data.User;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.plugin.Signature;
import com.fr.plugin.WPSConfig;
import com.fr.plugin.models.*;
import com.fr.plugin.utils.DbUtils;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;
import com.fr.plugin.utils.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.List;
import java.util.Map;


public class FileInfoApi extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return null;
    }


    @Override
    public String getPath() {
        return "/v1/3rd/file/info";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest request, HttpServletResponse httpServletResponse) throws Exception {
        WPSConfig instance = WPSConfig.getInstance();
        String token = request.getHeader("x-wps-weboffice-token");
        String fileId = request.getHeader("x-weboffice-file-id");
        String wSignature = WebUtils.getHTTPRequestParameter(request, "_w_signature");
        String userid = WebUtils.getHTTPRequestParameter(request, "_w_userid");
        if (!Signature.checkSig(request,wSignature)) {
            WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40005,"签名错误","请联系管理员"));
            return;
        }
        if (!HttpUtils.checkToken(token)) {
            WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40002, "token过期", "请联系管理员"));
            return;
        }
        if (StringUtils.isNotBlank(fileId)) {
            DbUtils db = new DbUtils(instance.getAppsecret());
            GetFileModel getFileModel = new GetFileModel();
            List<Map<String, String>> list = db.exQuery("select * from " + instance.getDomainName() + " where File_Id =?", fileId);
            if (list.isEmpty()) {
               WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40004,"文件不存在","请联系管理员"));
               return;
            }
            Map<String, String> map = list.get(0);
            String fileUrl = map.get("File_Url");
            String fileName = map.get("File_name");
            String creater = map.get("Fill_Name");
            if (StringUtils.isEmpty(fileUrl)) {
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40004,"文件保存路径不存在","请联系管理员"));
                return;
            }
            String filePath = fileUrl + fileName;
            File file = new File(filePath);
            if (!file.exists()) {
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40004,"文件已经被删除","请联系管理员"));
                return;
            }
            FileModel model = new FileModel();
            model.setId(fileId);
            model.setSize(file.length());
            model.setDownload_url(request,fileId);
            model.setName(map.get("File_name"));
            UserService userService = UserService.getInstance();
            User createUser = userService.getUserByUserName(creater);
            if (createUser == null) {
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(50001,"用户不存在","请联系管理员"));
                return;
            }
            UserModel userModel = new UserModel();
            userModel.setName(createUser.getRealName());
            userModel.setId(creater);
            getFileModel.setFile(model);
            getFileModel.setUser(userModel);
            WebUtils.printAsString(httpServletResponse, HttpUtils.obj2JsonStr(getFileModel));
        }else{
            WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(40004,"文件id不存在","请联系管理员"));
            return;
        }
    }
}
