package com.fr.plugin.api;

import com.fr.base.core.IgnoreBytesInputStream;
import com.fr.base.core.ParseResult;
import com.fr.base.core.PostParseUtils;
import com.fr.decision.fun.impl.BaseHttpHandler;
import com.fr.general.IOUtils;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.Signature;
import com.fr.plugin.WPSConfig;
import com.fr.plugin.models.UploadFileModel;
import com.fr.plugin.models.UploadFileWarpModel;
import com.fr.plugin.utils.DbUtils;
import com.fr.plugin.utils.ErrorUtils;
import com.fr.plugin.utils.HttpUtils;
import com.fr.stable.StringUtils;
import com.fr.third.springframework.web.bind.annotation.RequestMethod;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.io.FileOutputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Map;


public class FileSaveApi extends BaseHttpHandler {

    @Override
    public RequestMethod getMethod() {
        return null;
    }


    @Override
    public String getPath() {
        return "/v1/3rd/file/save";
    }

    @Override
    public boolean isPublic() {
        return true;
    }

    @Override
    public void handle(HttpServletRequest req, HttpServletResponse httpServletResponse) throws Exception {
        WPSConfig instance = WPSConfig.getInstance();
        InputStream inputStream = req.getInputStream();
        FineLoggerFactory.getLogger().info("访问文件保存接口");
        try {
            String token = req.getHeader("x-wps-weboffice-token");
            String fileId = req.getHeader("x-weboffice-file-id");
            String wSignature = WebUtils.getHTTPRequestParameter(req, "_w_signature");
            if (!Signature.checkSig(req, wSignature)) {
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "签名错误", "请联系管理员"));
                return;
            }
            if (!HttpUtils.checkToken(token)) {
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "token过期", "请联系管理员"));
                return;
            }
            if (StringUtils.isNotBlank(fileId)) {
                DbUtils db = new DbUtils(instance.getAppsecret());
                List<Map<String, String>> list = db.exQuery("select * from " + instance.getDomainName() + " where File_Id =?", fileId);
                if (list.isEmpty()) {
                    WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "保存的文件不存在", "请联系管理员"));
                    return;
                }
                Map<String, String> map = list.get(0);
                String fileUrl = map.get("File_Url");
                String fileName = map.get("File_name");
                if (StringUtils.isEmpty(fileUrl)) {
                    WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "保存的文件保存路径不存在", "请联系管理员"));
                    return;
                }
                String filePath = fileUrl + fileName;
                File file = new File(filePath);
                if (!file.exists()) {
                    WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "保存的文件已经被删除", "请联系管理员"));
                    return;
                }
                ParseResult parseResult = PostParseUtils.parse(inputStream, req.getCharacterEncoding());
                InputStream fileDecorator = new IgnoreBytesInputStream(inputStream, concat(this.concat(NEW_LINE_BYTES, parseResult.getBoundary().getBytes()), BOUNDARY_END));
                IOUtils.copyBinaryTo(fileDecorator, new FileOutputStream(file));
                UploadFileWarpModel uploadFileWarpModel = new UploadFileWarpModel();
                UploadFileModel uploadFileModel = new UploadFileModel();
                uploadFileModel.setId(fileId);
                uploadFileModel.setDownload_url(fileId);
                uploadFileModel.setName(fileName);
                uploadFileModel.setSize(file.length());
                uploadFileWarpModel.setFile(uploadFileModel);
                WebUtils.printAsString(httpServletResponse, HttpUtils.obj2JsonStr(uploadFileWarpModel));
                FineLoggerFactory.getLogger().info("文件保存成功 文件名:{}", fileName);
                return;
            } else {
                FineLoggerFactory.getLogger().info("文件保存失败 ");
                WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "要保存的文件不存在或找不到", "请联系管理员"));
                return;
            }
        } catch (Exception e) {
            FineLoggerFactory.getLogger().error("文件保存异常  ",e);
            WebUtils.printAsJSON(httpServletResponse, ErrorUtils.createError(101, "文件保存异常", "请联系管理员"));
            return;
        }
    }

    private byte[] concat(byte[] a, byte[] b) {
        byte[] c = new byte[a.length + b.length];
        System.arraycopy(a, 0, c, 0, a.length);
        System.arraycopy(b, 0, c, a.length, b.length);
        return c;
    }

    private static final int MAX_IMAGE_SIZE = 20971520;//20MB
    public static final String BASE64_DATA = "base64Data";
    private static final byte[] NEW_LINE_BYTES = new byte[]{13, 10};
    private static final byte[] BOUNDARY_END = new byte[]{45, 45};

}
